@@ -2,7 +2,7 @@
 
 #  ISC License
 #
-#  Copyright (c) 2020–2021, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
+#  Copyright (c) 2020–2022, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
 #
 #  Permission to use, copy, modify, and/or distribute this software for any
 #  purpose with or without fee is hereby granted, provided that the above
@@ -17,22 +17,12 @@
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 from __future__ import annotations
-from typing import Optional, Dict, Any, Callable
+from typing import Optional, Dict, Any, Callable, List
 from sty import fg
 from magneticalc.Debug import Debug
 from magneticalc.Field_Types import field_type_to_str
 from magneticalc.ModelAccess import ModelAccess
 
-# Note: Workaround for type hinting
-# noinspection PyUnreachableCode
-if False:
-    from magneticalc.Field import Field
-    from magneticalc.GUI import GUI
-    from magneticalc.Metric import Metric
-    from magneticalc.Parameters import Parameters
-    from magneticalc.SamplingVolume import SamplingVolume
-    from magneticalc.Wire import Wire
-
 
 class Model:
     """
@@ -50,7 +40,10 @@ class Model:
     # Used by L{Debug}
     DebugColor = fg.yellow
 
-    def __init__(self, gui: GUI) -> None:
+    def __init__(
+            self,
+            gui: GUI  # type: ignore
+    ) -> None:
         """
         Initializes the model.
 
@@ -59,27 +52,45 @@ class Model:
         Debug(self, ": Init")
         self.gui = gui
 
-        self.wire = None                                    # Set in L{set_wire}            via L{Wire_Widget}
-        self.sampling_volume = None                         # Set in L{set_sampling_volume} via L{SamplingVolume_Widget}
+        # Set in L{set_wire} via L{Wire_Widget}
+        self.wire: Optional[Wire] = None  # type: ignore
 
-        self.metric = None                                  # Set in L{set_metric}          via L{Metric_Widget}
-        self.parameters = None                              # Set in L{set_parameters}      via L{Parameters_Widget}
+        # Set in L{set_sampling_volume} via L{SamplingVolume_Widget}
+        self.sampling_volume: Optional[SamplingVolume] = None  # type: ignore
 
-        # Field cache and currently selected field
-        self._selected_field_type: Optional[int] = None     # Set in L{set_field}           via L{Field_Widget}
-        self._field_cache: Dict[int, Field] = {}            # Set in L{set_field}           via L{Field_Widget}
+        # Set in L{set_metric} via L{Metric_Widget}
+        self.metric: Optional[Metric] = None  # type: ignore
+
+        # Set in L{set_parameters} via L{Parameters_Widget}
+        self.parameters: Optional[Parameters] = None  # type: ignore
+
+        # Field cache and currently selected field:
+
+        # Set in L{set_field} via L{Field_Widget}
+        self._selected_field_type: Optional[int] = None
+
+        # Set in L{set_field} via L{Field_Widget}
+        self._field_cache: Dict[
+            int,
+            Field  # type: ignore
+        ] = {}
 
     @property
-    def field(self) -> Optional[Field]:
+    def field(
+            self
+    ) -> Optional[Field]:  # type: ignore
         """
         Returns the currently selected field if it is cached.
 
         @return: Field if cached, None otherwise
         """
-        return self._field_cache.get(self._selected_field_type, None)
+        return self._field_cache.get(self._selected_field_type, None) if self._selected_field_type is not None else None
 
     @field.setter
-    def field(self, field: Field) -> None:
+    def field(
+            self,
+            field: Field  # type: ignore
+    ) -> None:
         """
         Sets the currently selected field.
 
@@ -88,7 +99,10 @@ class Model:
         self._selected_field_type = field.get_type()
         self._field_cache[self._selected_field_type] = field
 
-    def get_valid_field(self, field_type: int) -> Optional[Field]:
+    def get_valid_field(
+            self,
+            field_type: int
+    ) -> Optional[Field]:  # type: ignore
         """
         Gets a field if it is cached and valid.
 
@@ -113,14 +127,23 @@ class Model:
 
         @return: String
         """
-        name_obj_map = {
-            "Wire": self.wire,
-            "SamplingVolume": self.sampling_volume,
-            "Field": self.field,
-            "Metric": self.metric,
-            "Parameters": self.parameters
+        return ", ".join(self.get_summary(valid=True))
+
+    def get_summary(self, valid: bool) -> List[str]:
+        """
+        Gets a list of (in)valid object names.
+
+        @param valid: True returns names of valid objects, False returns names of invalid objects
+        @return: List of strings
+        """
+        valid_map = {
+            "Wire"              : self.safe_valid(self.wire),
+            "Sampling Volume"   : self.safe_valid(self.sampling_volume),
+            "Field"             : self.safe_valid(self.field),
+            "Metric"            : self.safe_valid(self.metric),
+            "Parameters"        : self.safe_valid(self.parameters)
         }
-        return ", ".join([name for name, obj in name_obj_map.items() if self.safe_valid(obj)])
+        return [name for name, is_valid in valid_map.items() if valid == is_valid]
 
     def is_valid(self) -> bool:
         """
@@ -188,7 +211,11 @@ class Model:
                     self.wire.invalidate()
                     self.on_wire_invalid()
 
-    def set_wire(self, wire: Wire, invalidate_self: bool) -> None:
+    def set_wire(
+            self,
+            wire: Wire,  # type: ignore
+            invalidate_self: bool
+    ) -> None:
         """
         Sets the wire.
 
@@ -208,7 +235,11 @@ class Model:
             )
             self.wire = wire
 
-    def set_sampling_volume(self, sampling_volume: SamplingVolume, invalidate_self: bool) -> None:
+    def set_sampling_volume(
+            self,
+            sampling_volume: SamplingVolume,  # type: ignore
+            invalidate_self: bool
+    ) -> None:
         """
         Sets the sampling volume.
 
@@ -227,7 +258,11 @@ class Model:
             )
             self.sampling_volume = sampling_volume
 
-    def set_field(self, field: Field, invalidate_self: bool) -> None:
+    def set_field(
+            self,
+            field: Field,  # type: ignore
+            invalidate_self: bool
+    ) -> None:
         """
         Sets the field.
 
@@ -245,7 +280,11 @@ class Model:
             )
             self.field = field
 
-    def set_metric(self, metric: Metric, invalidate_self: bool) -> None:
+    def set_metric(
+            self,
+            metric: Metric,  # type: ignore
+            invalidate_self: bool
+    ) -> None:
         """
         Sets the metric.
 
@@ -262,7 +301,11 @@ class Model:
             )
             self.metric = metric
 
-    def set_parameters(self, parameters: Parameters, invalidate_self: bool) -> None:
+    def set_parameters(
+            self,
+            parameters: Parameters,  # type: ignore
+            invalidate_self: bool
+    ) -> None:
         """
         Sets the parameters.
 
@@ -289,7 +332,10 @@ class Model:
         """
         Debug(self, ".calculate_wire()")
 
+        assert self.wire is not None, "Attempting to calculate non-existing Wire"
+
         self.invalidate(do_sampling_volume=True, do_field=True, do_metric=True)
+
         return self.wire.recalculate(progress_callback)
 
     def calculate_sampling_volume(self, progress_callback: Callable) -> bool:
@@ -301,6 +347,8 @@ class Model:
         """
         Debug(self, ".calculate_sampling_volume()")
 
+        assert self.sampling_volume is not None, "Attempting to calculate non-existing SamplingVolume"
+
         self.invalidate(do_field=True, do_metric=True)
         return self.sampling_volume.recalculate(progress_callback)
 
@@ -314,6 +362,8 @@ class Model:
         """
         Debug(self, f".calculate_field(num_cores={num_cores})")
 
+        assert self.field is not None, "Attempting to calculate non-existing Field"
+
         self.invalidate(do_metric=True)
         return self.field.recalculate(self.wire, self.sampling_volume, progress_callback, num_cores)
 
@@ -326,6 +376,8 @@ class Model:
         """
         Debug(self, ".calculate_metric()")
 
+        assert self.metric is not None, "Attempting to calculate non-existing Metric"
+
         return self.metric.recalculate(self.sampling_volume, self.field, progress_callback)
 
     def calculate_parameters(self, progress_callback: Callable) -> bool:
@@ -337,6 +389,8 @@ class Model:
         """
         Debug(self, ".calculate_parameters()")
 
+        assert self.parameters is not None, "Attempting to calculate non-existing Parameters"
+
         return self.parameters.recalculate(self.wire, self.sampling_volume, self.field, progress_callback)
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -347,7 +401,7 @@ class Model:
         """
         Debug(self, ".on_wire_valid()")
 
-        self.gui.sidebar_left.wire_widget.update_labels()
+        self.gui.sidebar_left.wire_widget.update()
         self.gui.menu.update_wire_menu()
 
     def on_sampling_volume_valid(self) -> None:
@@ -374,7 +428,7 @@ class Model:
         """
         Debug(self, ".on_metric_valid()")
 
-        self.gui.sidebar_right.metric_widget.update_labels()
+        self.gui.sidebar_right.metric_widget.update()
 
         # The field labels are now created on-demand inside VispyCanvas.redraw()
         # self.gui.vispy_canvas.create_field_labels()
@@ -385,7 +439,7 @@ class Model:
         """
         Debug(self, ".on_parameters_valid()")
 
-        self.gui.sidebar_right.parameters_widget.update_labels()
+        self.gui.sidebar_right.parameters_widget.update()
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -395,7 +449,7 @@ class Model:
         """
         Debug(self, ".on_wire_invalid()")
 
-        self.gui.sidebar_left.wire_widget.update_labels()
+        self.gui.sidebar_left.wire_widget.update()
         self.gui.menu.update_wire_menu()
 
     def on_sampling_volume_invalid(self) -> None:
@@ -420,7 +474,7 @@ class Model:
         """
         Debug(self, ".on_metric_invalid()")
 
-        self.gui.sidebar_right.metric_widget.update_labels()
+        self.gui.sidebar_right.metric_widget.update()
         self.gui.vispy_canvas.delete_field_labels()
 
     def on_parameters_invalid(self) -> None:
@@ -429,4 +483,4 @@ class Model:
         """
         Debug(self, ".on_parameters_invalid()")
 
-        self.gui.sidebar_right.parameters_widget.update_labels()
+        self.gui.sidebar_right.parameters_widget.update()
