@@ -2,7 +2,7 @@
 
 #  ISC License
 #
-#  Copyright (c) 2020–2021, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
+#  Copyright (c) 2020–2022, Paul Wilhelm, M. Sc. <anfrage@paulwilhelm.de>
 #
 #  Permission to use, copy, modify, and/or distribute this software for any
 #  purpose with or without fee is hereby granted, provided that the above
@@ -19,12 +19,13 @@
 from __future__ import annotations
 from typing import Optional, List, Tuple
 import numpy as np
-import qtawesome as qta
-from PyQt5.QtCore import Qt
-from PyQt5.QtWidgets import QHBoxLayout, QLabel, QPushButton, QSpinBox, QComboBox, QSizePolicy, QWidget
+from PyQt5.QtWidgets import QHBoxLayout, QComboBox, QWidget, QSizePolicy
 from magneticalc.QGroupBox2 import QGroupBox2
 from magneticalc.QHLine import QHLine
 from magneticalc.QIconLabel import QIconLabel
+from magneticalc.QLabel2 import QLabel2
+from magneticalc.QPushButton2 import QPushButton2
+from magneticalc.QSpinBox2 import QSpinBox2
 from magneticalc.Comparison_Types import comparison_type_from_str
 from magneticalc.Constraint import Constraint
 from magneticalc.Constraint_Editor import Constraint_Editor
@@ -35,11 +36,6 @@ from magneticalc.OverridePadding_Dialog import OverridePadding_Dialog
 from magneticalc.SamplingVolume import SamplingVolume
 from magneticalc.Theme import Theme
 
-# Note: Workaround for type hinting
-# noinspection PyUnreachableCode
-if False:
-    from magneticalc.GUI import GUI
-
 
 class SamplingVolume_Widget(QGroupBox2):
     """ SamplingVolume_Widget class. """
@@ -48,8 +44,8 @@ class SamplingVolume_Widget(QGroupBox2):
     UnitsLabelWidth = 26
 
     # Spinbox limits
-    PaddingMin = -1e+3
-    PaddingMax = +1e+3
+    PaddingMin = -1000
+    PaddingMax = +1000
 
     # Resolution options
     ResolutionOptionsDict = {
@@ -72,7 +68,10 @@ class SamplingVolume_Widget(QGroupBox2):
         "1 / 256"   : -8,
     }
 
-    def __init__(self, gui: GUI) -> None:
+    def __init__(
+            self,
+            gui: GUI  # type: ignore
+    ) -> None:
         """
         Populates the widget.
 
@@ -90,47 +89,32 @@ class SamplingVolume_Widget(QGroupBox2):
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
         padding_icon_label = QIconLabel("Padding", "mdi.arrow-expand-all")
-        padding_override_button = QPushButton(" Override … ")
-        # noinspection PyUnresolvedReferences
-        padding_override_button.clicked.connect(self.override_padding)
-        padding_override_button.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
+        padding_override_button = QPushButton2("  Override …  ", "", self.override_padding)
         padding_icon_label.addWidget(padding_override_button)
-        padding_clear_button = QPushButton()
-        padding_clear_button.setIcon(qta.icon("fa.eraser"))
-        # noinspection PyUnresolvedReferences
-        padding_clear_button.clicked.connect(self.clear_padding)
-        padding_icon_label.addWidget(padding_clear_button)
-        padding_units_label = QLabel("cm")
-        padding_units_label.setAlignment(Qt.AlignRight)
-        padding_units_label.setFixedWidth(self.UnitsLabelWidth)
-        padding_icon_label.addWidget(padding_units_label)
+        padding_icon_label.addWidget(QPushButton2("", "fa.eraser", self.clear_padding))
+        padding_icon_label.addWidget(QLabel2("cm", align_right=True, width=self.UnitsLabelWidth))
         self.addLayout(padding_icon_label)
 
         self.padding_widget = QWidget()
         padding_layout = QHBoxLayout()
-        padding_label = [None, None, None]
-        self.padding_spinbox = [None, None, None]
+        self.padding_spinbox = []
         for i in range(3):
-            self.padding_spinbox[i] = QSpinBox(self.gui)
-            self.padding_spinbox[i].setMinimum(self.PaddingMin)
-            self.padding_spinbox[i].setMaximum(self.PaddingMax)
-            # noinspection PyUnresolvedReferences
-            self.padding_spinbox[i].valueChanged.connect(self.update_padding)
-            padding_label[i] = QLabel(["X", "Y", "Z"][i] + ":")
-            padding_label[i].setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
-            padding_layout.addWidget(padding_label[i], alignment=Qt.AlignVCenter)
-            padding_layout.addWidget(self.padding_spinbox[i], alignment=Qt.AlignVCenter)
+            padding_spinbox = QSpinBox2(
+                minimum=self.PaddingMin,
+                maximum=self.PaddingMax,
+                value=0,
+                value_changed=self.update_padding
+            )
+            self.padding_spinbox.append(padding_spinbox)
+            padding_layout.addWidget(QLabel2(["X", "Y", "Z"][i] + ":", expand=False))
+            padding_layout.addWidget(self.padding_spinbox[i])
         self.padding_widget.setLayout(padding_layout)
         self.addWidget(self.padding_widget)
 
         total_extent_layout = QHBoxLayout()
-        total_extent_label_left = QLabel("Total extent:")
-        total_extent_label_left.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
-        self.total_extent_label = QLabel("N/A")
-        self.total_extent_label.setStyleSheet(f"color: {Theme.MainColor};")
-        self.total_extent_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
-        total_extent_layout.addWidget(total_extent_label_left, alignment=Qt.AlignVCenter)
-        total_extent_layout.addWidget(self.total_extent_label, alignment=Qt.AlignVCenter)
+        self.total_extent_label = QLabel2("N/A", color=Theme.MainColor, align_right=True)
+        total_extent_layout.addWidget(QLabel2("Total extent:", italic=True, color=Theme.LiteColor))
+        total_extent_layout.addWidget(self.total_extent_label)
         self.addLayout(total_extent_layout)
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -138,25 +122,14 @@ class SamplingVolume_Widget(QGroupBox2):
         self.addWidget(QHLine())
 
         constraints_icon_label = QIconLabel("Constraints", "mdi.playlist-edit")
-        constraint_shortcut_label = QLabel("⟨F3⟩")
-        constraint_shortcut_label.setStyleSheet(f"font-size: 13px; color: {Theme.LiteColor}")
-        constraints_icon_label.addWidget(constraint_shortcut_label)
-
-        constraint_edit_button = QPushButton("Edit …")
-        # noinspection PyUnresolvedReferences
-        constraint_edit_button.clicked.connect(self.open_constraint_editor)
-        constraints_icon_label.addWidget(constraint_edit_button)
-
+        constraints_icon_label.addWidget(QLabel2("⟨F3⟩", font_size="13px", color=Theme.LiteColor, expand=False))
+        constraints_icon_label.addWidget(QPushButton2("Edit …", "", self.open_constraint_editor))
         self.addLayout(constraints_icon_label)
 
         total_constraints_layout = QHBoxLayout()
-        total_constraints_label_left = QLabel("Total constraints:")
-        total_constraints_label_left.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
-        self.total_constraints_label = QLabel("N/A")
-        self.total_constraints_label.setStyleSheet(f"color: {Theme.MainColor};")
-        self.total_constraints_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
-        total_constraints_layout.addWidget(total_constraints_label_left, alignment=Qt.AlignVCenter)
-        total_constraints_layout.addWidget(self.total_constraints_label, alignment=Qt.AlignVCenter)
+        self.total_constraints_label = QLabel2("N/A", color=Theme.MainColor, align_right=True)
+        total_constraints_layout.addWidget(QLabel2("Total constraints:", italic=True, color=Theme.LiteColor))
+        total_constraints_layout.addWidget(self.total_constraints_label)
         self.addLayout(total_constraints_layout)
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -166,11 +139,11 @@ class SamplingVolume_Widget(QGroupBox2):
         self.addLayout(QIconLabel("Resolution", "fa.th"))
 
         self.resolution_combobox = QComboBox()
+        self.resolution_combobox.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Minimum)
         resolution_layout = QHBoxLayout()
-        resolution_layout.addWidget(self.resolution_combobox, alignment=Qt.AlignVCenter)
-        self.resolution_units_label = QLabel(" Points / cm")
-        self.resolution_units_label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Expanding)
-        resolution_layout.addWidget(self.resolution_units_label, alignment=Qt.AlignVCenter)
+        resolution_layout.addWidget(self.resolution_combobox)
+        self.resolution_units_label = QLabel2(" Points / cm", expand=False)
+        resolution_layout.addWidget(self.resolution_units_label)
         self.addLayout(resolution_layout)
 
         # Populate resolution combobox
@@ -179,7 +152,7 @@ class SamplingVolume_Widget(QGroupBox2):
         # noinspection PyUnresolvedReferences
         self.resolution_combobox.currentIndexChanged.connect(
             lambda: self.set_sampling_volume(
-                resolution_exponent=self.ResolutionOptionsDict.get(
+                _resolution_exponent_=self.ResolutionOptionsDict.get(
                     self.resolution_combobox.currentText(),
                     0
                 )
@@ -187,13 +160,9 @@ class SamplingVolume_Widget(QGroupBox2):
         )
 
         total_points_layout = QHBoxLayout()
-        total_points_label_left = QLabel("Total sampling points:")
-        total_points_label_left.setStyleSheet(f"color: {Theme.LiteColor}; font-style: italic;")
-        self.total_points_label = QLabel("N/A")
-        self.total_points_label.setStyleSheet(f"color: {Theme.MainColor};")
-        self.total_points_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
-        total_points_layout.addWidget(total_points_label_left, alignment=Qt.AlignVCenter)
-        total_points_layout.addWidget(self.total_points_label, alignment=Qt.AlignVCenter)
+        self.total_points_label = QLabel2("N/A", color=Theme.MainColor, align_right=True)
+        total_points_layout.addWidget(QLabel2("Total sampling points:", italic=True, color=Theme.LiteColor))
+        total_points_layout.addWidget(self.total_points_label)
         self.addLayout(total_points_layout)
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -231,6 +200,8 @@ class SamplingVolume_Widget(QGroupBox2):
         # Initially load sampling volume from configuration
         self.set_sampling_volume(recalculate=False, invalidate_self=False)
 
+        self.update()
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def update_padding(self) -> None:
@@ -242,7 +213,7 @@ class SamplingVolume_Widget(QGroupBox2):
 
         Debug(self, ".update_padding()")
 
-        self.set_sampling_volume(padding=[self.padding_spinbox[i].value() for i in range(3)])
+        self.set_sampling_volume(_padding_=[self.padding_spinbox[i].value() for i in range(3)])
 
     def clear_padding(self) -> None:
         """
@@ -253,7 +224,7 @@ class SamplingVolume_Widget(QGroupBox2):
         self.gui.config.set_bool("sampling_volume_override_padding", False)
         self.gui.config.set_points("sampling_volume_bounding_box", [np.zeros(3), np.zeros(3)])
 
-        self.set_sampling_volume(padding=[0, 0, 0])
+        self.set_sampling_volume(_padding_=[0, 0, 0])
 
         self.blockSignals(True)
         for i in range(3):
@@ -276,36 +247,53 @@ class SamplingVolume_Widget(QGroupBox2):
         self.clear_padding()
 
         self.set_sampling_volume(
-            override_padding=True,
-            bounding_box=[
-                [dialog.bounds_min_spinbox[i].value() for i in range(3)],
-                [dialog.bounds_max_spinbox[i].value() for i in range(3)]
-            ]
+            _override_padding_=True,
+            _bounding_box_=(
+                np.array([dialog.bounds_min_spinbox[i].value() for i in range(3)]),
+                np.array([dialog.bounds_max_spinbox[i].value() for i in range(3)])
+            )
         )
 
         self.update_controls()
 
     # ------------------------------------------------------------------------------------------------------------------
 
+    def open_constraint_editor(self) -> None:
+        """
+        Opens the constraint editor.
+        Recalculates the sampling volume if the constraints changed.
+        """
+        Debug(self, ".open_constraint_editor()")
+
+        self.constraint_editor.show()
+
+        if self.constraint_editor.get_changed():
+
+            self.set_sampling_volume()
+
+            self.constraint_editor.clear_changed()
+
+    # ------------------------------------------------------------------------------------------------------------------
+
     def set_sampling_volume(
             self,
-            resolution_exponent: Optional[int] = None,
-            label_resolution_exponent: Optional[int] = None,
-            padding: Optional[List] = None,
-            override_padding: Optional[bool] = None,
-            bounding_box: List[List] = None,
+            _resolution_exponent_: Optional[int] = None,
+            _label_resolution_exponent_: Optional[int] = None,
+            _padding_: Optional[List] = None,
+            _override_padding_: Optional[bool] = None,
+            _bounding_box_: Optional[Tuple[np.ndarray, np.ndarray]] = None,
             recalculate: bool = True,
             invalidate_self: bool = True
     ) -> None:
         """
         Sets the sampling volume. This will overwrite the currently set sampling volume in the model.
-        Parameters may be left set to None in order to load their default value.
+        Any underscored parameter may be left set to None in order to load its default value.
 
-        @param resolution_exponent: Sampling volume resolution exponent
-        @param label_resolution_exponent: Sampling volume label resolution exponent
-        @param padding: Padding (3D point)
-        @param override_padding: Enable to override padding instead, setting the bounding box directly
-        @param bounding_box: Bounding box (used in conjunction with override_padding)
+        @param _resolution_exponent_: Sampling volume resolution exponent
+        @param _label_resolution_exponent_: Sampling volume label resolution exponent
+        @param _padding_: Padding (3D point)
+        @param _override_padding_: Enable to override padding instead, setting the bounding box directly
+        @param _bounding_box_: Bounding box (used in conjunction with override_padding)
         @param recalculate: Enable to trigger final re-calculation
         @param invalidate_self: Enable to invalidate the old sampling volume before setting a new one
         """
@@ -318,12 +306,12 @@ class SamplingVolume_Widget(QGroupBox2):
 
             resolution_exponent = self.gui.config.set_get_int(
                 "sampling_volume_resolution_exponent",
-                resolution_exponent
+                _resolution_exponent_
             )
 
             label_resolution_exponent = self.gui.config.set_get_int(
                 "sampling_volume_label_resolution_exponent",
-                label_resolution_exponent
+                _label_resolution_exponent_
             )
 
             resolution = np.power(2.0, resolution_exponent)
@@ -334,7 +322,7 @@ class SamplingVolume_Widget(QGroupBox2):
                 invalidate_self=invalidate_self
             )
 
-            self.readjust(padding, override_padding, bounding_box)
+            self.readjust(_padding_=_padding_, _override_padding_=_override_padding_, _bounding_box_=_bounding_box_)
 
             # Load sampling volume constraints from configuration
             for constraint in self.constraint_editor.get_constraints():
@@ -353,13 +341,17 @@ class SamplingVolume_Widget(QGroupBox2):
                 # The display widget depends on the sampling volume
                 self.gui.sidebar_right.display_widget.update()
 
+                # This forces updating the number of constraints if the sampling volume is already invalidated
+                if not self.gui.model.sampling_volume.is_valid():
+                    self.update()
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def readjust(
             self,
-            padding: Optional[List] = None,
-            override_padding: Optional[bool] = None,
-            bounding_box: Optional[Tuple[np.ndarray, np.ndarray]] = None,
+            _padding_: Optional[List] = None,
+            _override_padding_: Optional[bool] = None,
+            _bounding_box_: Optional[Tuple[np.ndarray, np.ndarray]] = None,
     ) -> None:
         """
         Readjusts the sampling volume to the currently set wire bounds and padding.
@@ -367,16 +359,16 @@ class SamplingVolume_Widget(QGroupBox2):
 
         Padding may also be overridden instead, setting the bounding box directly.
 
-        Parameters may be left set to None in order to load their default value.
+        Any underscored parameter may be left set to None in order to load its default value.
 
-        @param padding: Padding (3D point)
-        @param override_padding: Enable to override padding instead, setting the bounding box directly
-        @param bounding_box: Bounding box (used in conjunction with override_padding)
+        @param _padding_: Padding (3D point)
+        @param _override_padding_: Enable to override padding instead, setting the bounding box directly
+        @param _bounding_box_: Bounding box (used in conjunction with override_padding)
         """
         Debug(self, ".readjust()")
 
-        override_padding = self.gui.config.set_get_bool("sampling_volume_override_padding", override_padding)
-        bounding_box = self.gui.config.set_get_points("sampling_volume_bounding_box", bounding_box)
+        override_padding = self.gui.config.set_get_bool("sampling_volume_override_padding", _override_padding_)
+        bounding_box = self.gui.config.set_get_points("sampling_volume_bounding_box", _bounding_box_)
 
         if override_padding:
 
@@ -388,8 +380,9 @@ class SamplingVolume_Widget(QGroupBox2):
 
             bounds_min, bounds_max = self.gui.model.sampling_volume.get_bounds()
 
+            padding = self.gui.config.set_get_point("sampling_volume_padding", _padding_)
+
             # Adjust padding values if they are now outside the new minimum padding bounds
-            padding = self.gui.config.set_get_point("sampling_volume_padding", padding)
             padding_adjusted = False
             for i in range(3):
                 if bounds_min[i] > self.padding_spinbox[i].value():
@@ -414,17 +407,8 @@ class SamplingVolume_Widget(QGroupBox2):
         """
         Debug(self, ".update()")
 
-        self.update_controls()
         self.update_labels()
-
-    def update_controls(self) -> None:
-        """
-        Updates the controls.
-        """
-        Debug(self, ".update_controls()")
-
-        override_padding = self.gui.config.get_bool("sampling_volume_override_padding")
-        self.padding_widget.setEnabled(not override_padding)
+        self.update_controls()
 
     def update_labels(self) -> None:
         """
@@ -445,19 +429,12 @@ class SamplingVolume_Widget(QGroupBox2):
 
         self.total_constraints_label.setText(str(self.constraint_editor.get_count()))
 
-    # ------------------------------------------------------------------------------------------------------------------
-
-    def open_constraint_editor(self) -> None:
+    def update_controls(self) -> None:
         """
-        Opens the constraint editor.
-        Recalculates the sampling volume if the constraints changed.
+        Updates the controls.
         """
-        Debug(self, ".open_constraint_editor()")
+        Debug(self, ".update_controls()")
 
-        self.constraint_editor.show()
+        self.padding_widget.setEnabled(not self.gui.config.get_bool("sampling_volume_override_padding"))
 
-        if self.constraint_editor.get_changed():
-
-            self.set_sampling_volume()
-
-            self.constraint_editor.clear_changed()
+        self.indicate_valid(self.gui.model.sampling_volume is not None and self.gui.model.sampling_volume.is_valid())
